==> pan-0.13.3.tar.bz2: pan-0.13.3/pan/task-bodies.c <==
/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*********************
**********************  Includes
*********************/

#include <config.h>

#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include <pan/base/acache.h>
#include <pan/base/article.h>
#include <pan/base/debug.h>
#include <pan/base/log.h>
#include <pan/base/pan-glib-extensions.h>
#include <pan/base/pan-i18n.h>

#include <pan/nntp.h>
#include <pan/queue.h>
#include <pan/task-bodies.h>

/*********************
**********************  Defines / Enumerated types
*********************/

/*********************
**********************  Macros
*********************/

/*********************
**********************  Structures / Typedefs
*********************/

/*********************
**********************  Private Function Prototypes
*********************/

static int task_bodies_run (Task* item);

static char* task_bodies_describe (const StatusItem* item);

/*********************
**********************  Variables
*********************/

/***********
************  Extern
***********/

/***********
************  Public
***********/

/***********
************  Private
***********/

/*********************
**********************  BEGINNING OF SOURCE
*********************/

static void
task_bodies_destructor (PanObject * obj)
{
	TaskBodies * task;
	debug_enter ("task_bodies_destructor");

	/* sanity clause */
	g_return_if_fail (obj!=NULL);

	/* task-bodies dtor */
	task = TASK_BODIES(obj);
	acache_checkin (task->acache_key, (MessageIdentifier **)TASK(obj)->identifiers->pdata, TASK(obj)->identifiers->len);
	TASK_BODIES(obj)->working_index = 0;
	g_free (task->acache_key);

	/* destroy parent class */
	task_destructor (obj);

	debug_exit ("task_bodies_destructor");
}

/************
*************  PUBLIC ROUTINES
************/

PanObject*
task_bodies_new (Server               * server,
                 MessageIdentifier   ** identifiers,
                 int                    identifier_qty)
{
	int i;
	TaskBodies* bodies = NULL;
	gboolean needs_socket;
	const char * acache_key = ACACHE_DEFAULT_KEY;
	debug_enter ("task_bodies_new");

	/* sanity clause */
	g_return_val_if_fail (identifiers!=NULL, NULL);
	g_return_val_if_fail (identifier_qty!=0, NULL);

	/* check out all the articles */
	acache_checkout (acache_key, identifiers, identifier_qty);

	/* do we need a connection? */
	needs_socket = FALSE;
	for (i=0; i<identifier_qty && !needs_socket; ++i)
		if (!acache_has_message (acache_key, identifiers[i]->message_id))
			needs_socket = TRUE;

	/* create the object */
       	bodies = g_new0 (TaskBodies, 1);
        debug1 (DEBUG_PAN_OBJECT, "task_bodies_new: %p", identifiers);

	/* initialize the parent class */
	task_constructor (TASK(bodies), task_bodies_destructor,
	                                task_bodies_describe,
	                                task_bodies_run,
	                                server, TRUE,
	                                needs_socket);
	TASK(bodies)->gets_bodies = TRUE;
	TASK(bodies)->type = TASK_TYPE_BODIES;

	/* initialize the task-bodies */
	bodies->working_index = 0;
	bodies->acache_key = g_strdup (acache_key);
	task_add_identifiers (TASK(bodies), identifiers, identifier_qty);

	debug_exit ("task_bodies_new");
	return PAN_OBJECT(bodies);
}

PanObject*
task_bodies_new_from_articles (const Article       ** articles,
                               int                    article_qty)
{
	int i;
	MessageIdentifier ** mids;
	PanObject * retval;

	/* sanity clause */
	g_return_val_if_fail (articles!=NULL, NULL);
	g_return_val_if_fail (article_qty>0, NULL);
	g_return_val_if_fail (articles_are_valid (articles, article_qty), NULL);

	/* build the task */
	mids = g_alloca (sizeof(MessageIdentifier*) * article_qty);
	for (i=0; i<article_qty; ++i)
		mids[i] = message_identifier_new_from_article (articles[i]);
	retval = task_bodies_new (articles[0]->group->server, mids, article_qty);
	for (i=0; i<article_qty; ++i)
		g_object_unref (mids[i]);

	/* cleanup */
	return retval;
}

/*****
******
*****/

static char*
task_bodies_describe (const StatusItem* si)
{
	char * retval;
	GPtrArray * mids = TASK(si)->identifiers;

	if (mids->len == 1u) {
		MessageIdentifier * mid = MESSAGE_IDENTIFIER (g_ptr_array_index (mids, 0));
		retval = g_strdup (message_identifier_get_readable_name (mid));
	} else {
		retval = g_strdup_printf (_("Downloading %u articles"), mids->len);
	}

	return retval;
}

/*****
******
*****/

static int
task_bodies_run (Task* task)
{
	int retval;
	int steps = (int) task->identifiers->len;
	debug_enter ("task_bodies_run");

	if (!task->needs_socket)
	{
		retval = TASK_SUCCESS;
	}
	else
	{
		status_item_emit_init_steps (STATUS_ITEM(task), steps);

		retval = nntp_download_bodies (STATUS_ITEM(task),
					       task->sock,
					       &task->hint_abort,
					       (MessageIdentifier **)task->identifiers->pdata,
					       task->identifiers->len,
					       &TASK_BODIES(task)->working_index,
					       FALSE);

		if (retval == TASK_SUCCESS)
			log_add_va (LOG_INFO, ("Fetched %u bodies"), steps);
	}

	debug_exit ("task_bodies_run");
	return retval;
}

==> pan-0.13.4.tar.bz2: pan-0.13.4/pan/task-bodies.c <==
/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*********************
**********************  Includes
*********************/

#include <config.h>

#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include <pan/base/acache.h>
#include <pan/base/article.h>
#include <pan/base/debug.h>
#include <pan/base/log.h>
#include <pan/base/pan-glib-extensions.h>
#include <pan/base/pan-i18n.h>

#include <pan/nntp.h>
#include <pan/queue.h>
#include <pan/task-bodies.h>

/*********************
**********************  Defines / Enumerated types
*********************/

/*********************
**********************  Macros
*********************/

/*********************
**********************  Structures / Typedefs
*********************/

/*********************
**********************  Private Function Prototypes
*********************/

static void task_bodies_run (Task* item, PanSocket * sock);

static char* task_bodies_describe (const StatusItem* item);

/*********************
**********************  Variables
*********************/

/***********
************  Extern
***********/

/***********
************  Public
***********/

/***********
************  Private
***********/

/*********************
**********************  BEGINNING OF SOURCE
*********************/

static void
task_bodies_destructor (PanObject * obj)
{
	TaskBodies * task;
	gpointer p;
	debug_enter ("task_bodies_destructor");

	/* sanity clause */
	g_return_if_fail (obj!=NULL);

	/* task-bodies dtor */
	task = TASK_BODIES(obj);
	acache_checkin (task->acache_key, (MessageIdentifier **)TASK(obj)->identifiers->pdata, TASK(obj)->identifiers->len);
	g_free (task->acache_key);
	while ((p = g_queue_pop_head (task->download_mids)))
		g_object_unref (G_OBJECT(p));
	g_queue_free (task->download_mids);

	/* destroy parent class */
	task_destructor (obj);

	debug_exit ("task_bodies_destructor");
}

/************
*************  PUBLIC ROUTINES
************/

PanObject*
task_bodies_new (Server               * server,
                 MessageIdentifier   ** mids,
                 int                    mid_qty)
{
	int i;
	int lines_total = 0;
	int lines_cached = 0;
	TaskBodies * bodies = NULL;
	const char * acache_key = ACACHE_DEFAULT_KEY;
	debug_enter ("task_bodies_new");

	/* sanity clause */
	g_return_val_if_fail (mids!=NULL, NULL);
	g_return_val_if_fail (mid_qty!=0, NULL);

	/* check out all the articles */
	acache_checkout (acache_key, mids, mid_qty);

	/* create the object */
       	bodies = g_new0 (TaskBodies, 1);
        debug1 (DEBUG_PAN_OBJECT, "task_bodies_new: %p", mids);

	/* initialize the parent class' bits */
	task_constructor (TASK(bodies), task_bodies_destructor, task_bodies_describe, server, TRUE);
	TASK(bodies)->gets_bodies = TRUE;
	TASK(bodies)->type = TASK_TYPE_BODIES;
	task_add_identifiers (TASK(bodies), mids, mid_qty);

	/* initialize the task_bodies bits */
	bodies->acache_key = g_strdup (acache_key);
	bodies->download_mids = g_queue_new ();
	for (i=0; i<mid_qty; ++i) {
		lines_total += mids[i]->line_qty;
		if (acache_has_message (acache_key, mids[i]->message_id))
			lines_cached += mids[i]->line_qty;
		else {
			g_queue_push_tail (bodies->download_mids, mids[i]);
			g_object_ref (mids[i]);
		}
	}
	bodies->download_mids_qty = bodies->download_mids->length;

	/* set the state */
	if (g_queue_is_empty (bodies->download_mids))
		task_state_set (&TASK(bodies)->state, TASK_SUCCESS);
	else
		task_state_set_need_socket (&TASK(bodies)->state, server, task_bodies_run);

	/* init the status-item */
	status_item_emit_init_steps (STATUS_ITEM(bodies), lines_total);
	status_item_emit_set_step (STATUS_ITEM(bodies), lines_cached);
	if (mid_qty > 1)
		status_item_emit_status_va (STATUS_ITEM(bodies), _("Downloading %d"), mid_qty);
	else
		status_item_emit_status_va (STATUS_ITEM(bodies), _("\"%s\""), message_identifier_get_readable_name (mids[0]));

	/* return the task */
	debug_exit ("task_bodies_new");
	return PAN_OBJECT(bodies);
}

PanObject*
task_bodies_new_from_articles (const Article       ** articles,
                               int                    article_qty)
{
	int i;
	MessageIdentifier ** mids;
	PanObject * retval;

	/* sanity clause */
	g_return_val_if_fail (articles!=NULL, NULL);
	g_return_val_if_fail (article_qty>0, NULL);
	g_return_val_if_fail (articles_are_valid (articles, article_qty), NULL);

	/* build the task */
	mids = g_alloca (sizeof(MessageIdentifier*) * article_qty);
	for (i=0; i<article_qty; ++i)
		mids[i] = message_identifier_new_from_article (articles[i]);
	retval = task_bodies_new (articles[0]->group->server, mids, article_qty);
	for (i=0; i<article_qty; ++i)
		g_object_unref (mids[i]);

	/* cleanup */
	return retval;
}

/*****
******
*****/

static char*
task_bodies_describe (const StatusItem* si)
{
	char * retval;
	GPtrArray * mids = TASK(si)->identifiers;

	if (mids->len != 1u)
		retval = g_strdup_printf (_("Downloading %u articles"), mids->len);
	else {
		MessageIdentifier * mid = MESSAGE_IDENTIFIER (g_ptr_array_index (mids, 0));
		retval = g_strdup (message_identifier_get_readable_name (mid));
	}

	return retval;
}

/*****
******
*****/

static void
task_bodies_run (Task * task, PanSocket * sock)
{
	int result;
	TaskBodies * bodies = TASK_BODIES(task);
	MessageIdentifier * mid;
	debug_enter ("task_bodies_run");

	mid = g_queue_pop_head (bodies->download_mids);

	/* before we do work, let the queue know if we need another socket */
	if (!g_queue_is_empty (bodies->download_mids)) {
		task_state_set_need_socket (&task->state, task->server, task_bodies_run);
		queue_wakeup ();
	}

	/* download the article */	
	result = nntp_article_download (STATUS_ITEM(task), sock, mid, &task->hint_abort, NNTP_VERBOSE_NEXT_STEP);

	/* if we got it, or if we'll never get it, remove it from the list.
	 * otherwise maybe we lost a connection;
	 * save the mid for a future attempt */
	if (result == TASK_SUCCESS || result == TASK_FAIL_HOPELESS) 
	{
		g_object_unref (G_OBJECT(mid));

		if (!--bodies->download_mids_qty)
			task_state_set (&task->state, TASK_SUCCESS);
	}
	else
	{
		g_queue_push_tail (bodies->download_mids, mid);
		task_state_set_need_socket (&task->state, task->server, task_bodies_run);
	}

	debug_exit ("task_bodies_run");
}
